@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: apr 13 2022 (10:06) 
 ## Version: 
-## Last-Updated: May 30 2022 (00:40) 
+## Last-Updated: Oct 17 2022 (11:38) 
 ##           By: Brice Ozenne
-##     Update #: 314
+##     Update #: 370
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -41,27 +41,35 @@
     ## ** identify unique patterns
 
     ## *** var
-    lpObs.var <- interaction(as.data.frame(X.var), drop = TRUE, sep = sep[1])
-    if(!all(levels(lpObs.var) %in% Ulp.var)){
-        warning("Something went wrong when extracting the variance patterns. \n")
-    }
-    lpnObs.var <- as.numeric(factor(lpObs.var, levels = Ulp.var))
-    lpnCluster.var <- stats::setNames(sapply(U.cluster, function(iC){
-        paste(lpnObs.var[index.cluster[[iC]]], collapse=".")
-    }), U.cluster)
+    if(NCOL(X.var)>0){
+        lpObs.var <- interaction(as.data.frame(X.var), drop = TRUE, sep = sep[1])
+        if(!all(levels(lpObs.var) %in% Ulp.var)){
+            warning("Something went wrong when extracting the variance patterns. \n")
+        }
+        lpnObs.var <- as.numeric(factor(lpObs.var, levels = Ulp.var))
+        lpnCluster.var <- stats::setNames(sapply(U.cluster, function(iC){
+            paste(lpnObs.var[index.cluster[[iC]]], collapse=".")
+        }), U.cluster)
 
-    name.pattern.var <- sort(unique(lpnCluster.var))
-    cluster.pattern.var <- which(!duplicated(lpnCluster.var))
-    cluster.pattern.var <- cluster.pattern.var[match(lpnCluster.var[cluster.pattern.var], name.pattern.var)]
-    n.pattern.var <- length(name.pattern.var)
+        name.pattern.var <- sort(unique(lpnCluster.var))
+        cluster.pattern.var <- which(!duplicated(lpnCluster.var))
+        cluster.pattern.var <- cluster.pattern.var[match(lpnCluster.var[cluster.pattern.var], name.pattern.var)]
+        n.pattern.var <- length(name.pattern.var)
+    }
 
     ## *** cor
     if(!is.null(X.cor)){
         lpObs.cor <- interaction(as.data.frame(X.cor), drop = TRUE, sep = sep[1])
-        if(!all(levels(lpObs.cor) %in% sort(Ulp.cor))){
-            warning("Something went wrong when extracting the correlation patterns. \n")
+
+        if(!is.null(Ulp.cor)){
+            if(!all(levels(lpObs.cor) %in% sort(Ulp.cor))){
+                warning("Something went wrong when extracting the correlation patterns. \n")
+            }
+            lpnObs.cor <- as.numeric(factor(lpObs.cor, levels = Ulp.cor))
+        }else{
+            lpnObs.cor <- as.numeric(lpObs.cor)
         }
-        lpnObs.cor <- as.numeric(factor(lpObs.cor, levels = Ulp.cor))
+
         lpnCluster.cor0 <- stats::setNames(lapply(U.cluster, function(iC){
             if(length(index.cluster[[iC]])>1){
                 return(lpnObs.cor[index.cluster[[iC]]])
@@ -71,7 +79,7 @@
         }), U.cluster)
         lpnCluster.cor <- sapply(lpnCluster.cor0, paste, collapse = ".")
         name.pattern.cor <- unique(lpnCluster.cor)
-        if(!heterogeneous){
+        if(!is.character(heterogeneous) && !heterogeneous){
             name.pattern.cor <- name.pattern.cor[order(nchar(as.character(name.pattern.cor)))]
         }else{
             name.pattern.cor <- sort(name.pattern.cor)
@@ -85,11 +93,46 @@
 
     ## *** var and cor
     if(!is.null(X.cor)){
-        lpnCluster <- paste(as.numeric(factor(lpnCluster.var, name.pattern.var)),
-                            as.numeric(factor(lpnCluster.cor, name.pattern.cor)),
-                            sep = sep[1])       
-    }else{
+        if(NCOL(X.var)>0){
+            lpnCluster <- paste(as.numeric(factor(lpnCluster.var, name.pattern.var)),
+                                as.numeric(factor(lpnCluster.cor, name.pattern.cor)),
+                                sep = sep[1])
+        }else{
+            lpnCluster <- as.numeric(factor(lpnCluster.cor, name.pattern.cor))
+            name.pattern.var <- NULL
+        }
+    }else if(NCOL(X.var)>0){
         lpnCluster <- as.character(as.numeric(factor(lpnCluster.var, name.pattern.var)))
+    }else{ ## no variance nor correlation structure
+        pattern.cluster <- data.frame(index.cluster = names(index.cluster),
+                                      pattern = sapply(index.cluster,length),
+                                      var = paste0("length",sapply(index.cluster,length)))
+        Upattern.nRep <- unique(pattern.cluster$pattern)        
+        Upattern.name <- paste0("length",Upattern.nRep)
+
+        structure$X$Upattern <- data.frame(name = 1:length(Upattern.nRep),
+                                           var = Upattern.name,
+                                           cor = NA,
+                                           index.strata = 1,
+                                           n.time = Upattern.nRep,
+                                           time = NA,
+                                           param  = NA,
+                                           index.cluster = NA,
+                                           n.cluster = table(pattern.cluster$var)[Upattern.name])
+        structure$X$Upattern$param <- list(NULL)
+        structure$X$Upattern$index.cluster <- tapply(pattern.cluster$index.cluster,pattern.cluster$var,function(x){x}, simplify = FALSE)[Upattern.name]
+        structure$X$Upattern$time <- lapply(structure$X$Upattern$index.cluster,function(iC){index.clusterTime[[iC[1]]]})
+        rownames(structure$X$Upattern) <- NULL
+
+        structure$X$Xpattern.var <- stats::setNames(lapply(Upattern.name, function(iPattern){ ## iPattern <- Upattern.name[1]
+            iM <- matrix(nrow = Upattern.nRep[iPattern==Upattern.name], ncol = 0)
+            attr(iM,"index.cluster") <- pattern.cluster$index.cluster[pattern.cluster$var==iPattern]
+            attr(iM,"index.time") <- index.clusterTime[[attr(iM,"index.cluster")[1]]]
+            return(iM)
+        }),Upattern.name)
+        structure$X$Xpattern.cor <- NULL
+        structure$X$pattern.cluster <- pattern.cluster
+        return(structure)
     }
     test.duplicated <- duplicated(lpnCluster)
     name.pattern <- lpnCluster[!test.duplicated]
@@ -98,27 +141,35 @@
     n.pattern <- length(name.pattern)
 
     pattern.cluster <- data.frame(index.cluster = U.cluster,
-                                      pattern = factor(name.pattern[match(lpnCluster, name.pattern)], levels = name.pattern))
+                                  pattern = factor(name.pattern[match(lpnCluster, name.pattern)], levels = name.pattern))
 
     ## *** var/cor pattern corresponding to pattern
-    pattern.indexVar <- lpnCluster.var[!test.duplicated][order.pattern]
+    if(NCOL(X.var)>0){
+        pattern.indexVar <- lpnCluster.var[!test.duplicated][order.pattern]
+    }else{
+        pattern.indexVar <- NA
+    }
     if(!is.null(X.cor)){
         pattern.indexCor <- lpnCluster.cor[!test.duplicated][order.pattern]
     }else{
         pattern.indexCor <- NA
     }
     Upattern <- data.frame(name = name.pattern,
-                               var = pattern.indexVar,
-                               cor = pattern.indexCor,
-                               index.strata = NA,
-                               n.time = NA,
-                               param = NA,
-                               index.cluster = NA,
-                               n.cluster = NA)
+                           var = pattern.indexVar,
+                           cor = pattern.indexCor,
+                           index.strata = NA,
+                           time = NA,
+                           n.time = NA,
+                           param = NA,
+                           index.cluster = NA,
+                           n.cluster = NA)
 
     Upattern$index.cluster <- stats::setNames(lapply(name.pattern, function(iPattern){which(lpnCluster == iPattern)}), name.pattern)
     Upattern$n.cluster <- sapply(Upattern$index.cluster, length)
-    Upattern$n.time <- sapply(Upattern$index.cluster, function(iId){length(index.clusterTime[[iId[1]]])})
+    Upattern$time <- lapply(Upattern$index.cluster, function(iId){
+        index.clusterTime[[iId[1]]]
+    })
+    Upattern$n.time <- sapply(Upattern$time,length)
     Upattern$index.strata <- sapply(Upattern$index.cluster, function(iId){unname(unique(index.clusterStrata[iId]))}, simplify = FALSE)
     attr(Upattern, "level.var") <- name.pattern.var
     attr(Upattern, "level.cor") <- name.pattern.cor
@@ -143,9 +194,6 @@
         iParam.sigma <- intersect(names(iIndex.param),param.sigma)
         
         attr(iX,"index.cluster") <- iC.all
-        if(sum(!duplicated(index.clusterTime[iC.all]))==1){
-            attr(iX,"index.time") <- index.clusterTime[[iC]]
-        }
         attr(iX,"index.strata") <- unname(index.clusterStrata[iC])
         attr(iX,"param") <- names(iIndex.param)
         attr(iX,"Mindicator.param") <- stats::setNames(lapply(iIndex.param,function(iCol){tcrossprod(iX[,iCol],rep(1,NROW(iX))) + t(tcrossprod(iX[,iCol],rep(1,NROW(iX))))}),
@@ -171,7 +219,7 @@
         ls.pair <- vector(mode = "list", length = max(iN.pair))
 
         ls.pair[iN.pair[iN.pair>0]] <- lapply(iN.pair[iN.pair>0], function(iN){.unorderedPairs(1:iN, distinct = TRUE)})
-        Xpattern.cor <- stats::setNames(lapply(setdiff(name.pattern.cor,""),function(iP){ ## iP <- name.pattern.cor[1]
+        Xpattern.cor <- stats::setNames(lapply(setdiff(name.pattern.cor,""),function(iP){ ## iP <- name.pattern.cor[2]
             ## ignore pattern "", i.e. patterns with no pair
             iC.all <- pattern.cluster$index.cluster[which(pattern.cluster$cor==iP)]
             iC <- iC.all[1]
@@ -179,10 +227,13 @@
             iC.code <- lpnCluster.cor0[[iC]]
             iC.pair <- ls.pair[[length(iC.code)]]
             iC.codepair <- matrix(iC.code[iC.pair], ncol = 2, byrow = TRUE, dimnames = list(NULL,c("lp.x","lp.y")))
-            iC.lp.xy <- paste0(attr(param,"level.cor")[iC.codepair[,"lp.x"]],sep[2],attr(param,"level.cor")[iC.codepair[,"lp.y"]])
-            iC.param <- names(lp.xy)[match(iC.lp.xy,lp.xy)] ## can contain NA when asking a new structure without having seen a particular pair
+            if(!is.null(attr(param,"level.cor"))){
+                iC.lp.xy <- paste0(attr(param,"level.cor")[iC.codepair[,"lp.x"]],sep[2],attr(param,"level.cor")[iC.codepair[,"lp.y"]])
+                iC.param <- names(lp.xy)[match(iC.lp.xy,lp.xy)] ## can contain NA when asking a new structure without having seen a particular pair
+            }else{
+                iC.param <- paste(param.rho[param.rho$strata == index.clusterStrata[iC],"name"], collapse = "|")
+            }
             ## unique(iC.lp.xy)
-
             iC.table <- rbind(data.frame(row = iC.pair[1,], col = iC.pair[2,], param = iC.param),
                               data.frame(row = iC.pair[2,], col = iC.pair[1,], param = iC.param))
             ## iX.pairwise <- matrix(0, nrow = NROW(iC.table), ncol = length(param.rho$name),
@@ -241,9 +292,15 @@
 ## * .findUpatterns.CS
 .findUpatterns.CS <- .findUpatterns.ID
 
+## * .findUpatterns.TOEPLITZ
+.findUpatterns.TOEPLITZ <- .findUpatterns.ID
+
 ## * .findUpatterns.UN
 .findUpatterns.UN <- .findUpatterns.ID
 
+## * .findUpatterns.EXP
+.findUpatterns.EXP <- .findUpatterns.ID
+
 ## * .findUpatterns_CUSTOM
 .findUpatterns.CUSTOM <- function(structure, 
                                   index.clusterTime, U.time,
@@ -316,17 +373,19 @@
         pattern.indexCor <- NA
     }
     Upattern <- data.frame(name = name.pattern,
-                               var = pattern.indexVar,
-                               cor = pattern.indexCor,
-                               index.strata = NA,
-                               n.time = NA,
-                               param = NA,
-                               index.cluster = NA,
-                               n.cluster = NA)
+                           var = pattern.indexVar,
+                           cor = pattern.indexCor,
+                           index.strata = NA,
+                           n.time = NA,
+                           time = NA,
+                           param = NA,
+                           index.cluster = NA,
+                           n.cluster = NA)
 
     Upattern$index.cluster <- stats::setNames(lapply(name.pattern, function(iPattern){which(lpnCluster == iPattern)}), name.pattern)
     Upattern$n.cluster <- sapply(Upattern$index.cluster, length)
     Upattern$n.time <- sapply(Upattern$index.cluster, function(iId){length(index.clusterTime[[iId[1]]])})
+    Upattern$time <- lapply(Upattern$index.cluster,function(iC){index.clusterTime[[iC[1]]]})
     Upattern$index.strata <- sapply(Upattern$index.cluster, function(iId){unname(unique(index.clusterStrata[iId]))}, simplify = FALSE)
     attr(Upattern, "level.var") <- name.pattern.var
     attr(Upattern, "level.cor") <- name.pattern.cor
@@ -344,9 +403,6 @@
         iC <- iC.all[1]
         iX <- X.var[index.cluster[[iC]],,drop=FALSE]
         attr(iX,"index.cluster") <- iC.all
-        if(sum(!duplicated(index.clusterTime[iC.all]))==1){
-            attr(iX,"index.time") <- index.clusterTime[[iC]]
-        }
         attr(iX,"index.strata") <- unname(index.clusterStrata[iC])
         return(iX)
     }),name.pattern.var)
