@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: mar  5 2021 (21:50) 
 ## Version: 
-## Last-Updated: jun 24 2022 (15:03) 
+## Last-Updated: jan  3 2023 (18:39) 
 ##           By: Brice Ozenne
-##     Update #: 2305
+##     Update #: 2385
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -18,12 +18,15 @@
 
 ## * model.matrix.lmm (code)
 ##' @export
-model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies = TRUE, ...){
+model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies = TRUE, drop.X = NULL, ...){
 
     ## ** normalize user imput
     if(identical(effects,"all")){
         effects <- c("mean","variance")
     }
+    if(is.null(drop.X)){
+        drop.X <- LMMstar.options("drop.X")
+    }
 
     effects <- match.arg(effects, c("mean","variance"), several.ok = TRUE)
     dots <- list(...)
@@ -178,7 +181,7 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
                                   pair.varcoef = object$design$vcov$X$pair.varcoef)
 
             ## form design matrix    
-            outDesign <- .vcov.matrix.lmm(structure = object$design$vcov, data = data.var, index.cluster = outInit$index.cluster)
+            outDesign <- .vcov.matrix.lmm(structure = object$design$vcov, data = data.var, index.cluster = outInit$index.cluster, drop.X = drop.X)
             design$vcov$X$var <- outDesign$var
             design$vcov$X$cor <- outDesign$cor
             ## handle the case where structure is UN even though each cluster contain a single observation
@@ -216,7 +219,7 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
 
 ## * .mean.matrix.lmm
 .mean.matrix.lmm <- function(formula, colnames, data,
-                             stratify, name.strata, U.strata){
+                             stratify, name.strata, U.strata, drop.X){
 
 
     ## ** design matrix
@@ -226,7 +229,7 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
         ## *** generate design matrix for each strata
         ls.X.mean <- lapply(U.strata, function(iS){ ## iS <- U.strata[1]
             if(is.null(colnames)){
-                iX <- .model.matrix_regularize(formula, data = data[data$XXstrataXX==iS,,drop=FALSE], type = "mean")
+                iX <- .model.matrix_regularize(formula, data = data[data$XXstrataXX==iS,,drop=FALSE], type = "mean", drop.X = drop.X)
             }else{
                 iX <- stats::model.matrix(formula, data[data$XXstrataXX==iS,,drop=FALSE])
             }
@@ -259,7 +262,7 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
         }
     }else{
         if(is.null(colnames)){
-            X.mean <- .model.matrix_regularize(formula, data = data, type = "mean")
+            X.mean <- .model.matrix_regularize(formula, data = data, type = "mean", drop.X = drop.X)
             strata.mu <- stats::setNames(rep(U.strata,NCOL(X.mean)), colnames(X.mean))
         }else{
             X.mean <-  stats::model.matrix(formula, data)[,colnames,drop=FALSE]
@@ -276,7 +279,7 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
 
 ## * .vcov.matrix.lmm
 ## output observation specific design matrix (but no covariance pattern)
-.vcov.matrix.lmm <- function(structure, data, index.cluster){
+.vcov.matrix.lmm <- function(structure, data, index.cluster, drop.X){
 
     cluster.var <- structure$name$cluster
     time.var <- structure$name$time
@@ -311,27 +314,15 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
 
     ## data
     dataVar <- data
-    if(length(all.vars(formula.var))>0 && structure$type %in% c("ID","IND","CS","UN")){
+    if(length(all.vars(formula.var))>0 && structure$type %in% c("ID","IND","CS","UN","TOEPLITZ")){
         for(iVar in all.vars(formula.var)){
-            if(heterogeneous == FALSE){
-                if(iVar == strata.var){
-                    dataVar[[iVar]] <- as.factor(data[[iVar]])
-                }else if(is.logical(data[[iVar]])){
-                    dataVar[[iVar]] <- as.numeric(data[[iVar]]) + 1
-                }else if(!is.numeric(data[[iVar]])){
-                    dataVar[[iVar]] <- as.numeric(as.factor(data[[iVar]]))
-                }else if(is.numeric(data[[iVar]])){
-                    dataVar[[iVar]] <- data[[iVar]] - min(data[[iVar]]) + 1
-                }
-            }else if(heterogeneous){
-                dataVar[[iVar]] <- as.factor(data[[iVar]])
-            }
+            dataVar[[iVar]] <- as.factor(data[[iVar]])
         }
     }
     dataCor <- data
-    if(length(all.vars(formula.cor))>0 && structure$type %in% c("ID","IND","CS","UN")){
+    if(length(all.vars(formula.cor))>0 && structure$type %in% c("ID","IND","CS","UN","TOEPLITZ")){
         for(iVar in all.vars(formula.cor)){
-            if(heterogeneous == FALSE){
+            if(structure$type=="TOEPLITZ" || heterogeneous == FALSE){
                 if(iVar == strata.var){
                     dataCor[[iVar]] <- as.factor(data[[iVar]])
                 }else if(is.logical(data[[iVar]])){
@@ -349,10 +340,10 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
     ## ** design matrix
     out <- list(var = NULL, cor = NULL, xfactor = list(var = NULL, cor = NULL))
     if(is.null(structure$param)){ ## structure
-        out$var <- .colnameOrder(.model.matrix_regularize(formula.var, data = dataVar, augmodel = TRUE, type = "variance"), strata.var = strata.var, n.strata = n.strata)
+        out$var <- .colnameOrder(.model.matrix_regularize(formula.var, data = dataVar, augmodel = TRUE, type = "variance", drop.X = drop.X), strata.var = strata.var, n.strata = n.strata)
         out$xfactor$var <- stats::.getXlevels(stats::terms(formula.var),stats::model.frame(formula.var,dataVar))
         if(!is.null(formula.cor) && n.time>1 && any(sapply(index.cluster,length)>1)){  ## at least one individual with more than timepoint
-            out$cor <- .colnameOrder(.model.matrix_regularize(formula.cor, data = dataCor, augmodel = TRUE, type = "correlation"), strata.var = strata.var, n.strata = n.strata)
+            out$cor <- .colnameOrder(.model.matrix_regularize(formula.cor, data = dataCor, augmodel = TRUE, type = "correlation", drop.X = drop.X), strata.var = strata.var, n.strata = n.strata)
             out$xfactor$cor <- stats::.getXlevels(stats::terms(formula.cor),stats::model.frame(formula.cor,dataCor)) 
         }
     }else{ ## newdata        
@@ -378,7 +369,7 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
 ## * .model.matrix.lmm
 .model.matrix.lmm <- function(formula.mean, structure,
                               data, var.outcome, var.weights,
-                              stratify.mean,
+                              stratify.mean, drop.X,
                               precompute.moments){
 
     ## ** indexes
@@ -394,7 +385,7 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
     ## use stats::model.frame to handle splines
     data.mf <- stats::model.frame(stats::update(formula.mean,~.+XXindexXX+XXtimeXX+XXclusterXX+XXstrataXX),data)
     X.mean <- .mean.matrix.lmm(formula = formula.mean, colnames = NULL, data = data.mf,
-                               stratify = stratify.mean, name.strata = structure$name$strata, U.strata = U.strata)  ## only stratify mean if gls optimizer
+                               stratify = stratify.mean, name.strata = structure$name$strata, U.strata = U.strata, drop.X = drop.X)  ## only stratify mean if gls optimizer
     strata.mu <- attr(X.mean,"strata.mu")
     attr(X.mean,"strata.mu") <- NULL
     attr(X.mean,"terms") <- attr(data.mf,"terms")
@@ -402,7 +393,7 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
     ## ** variance
 
     ## *** design matrix
-    outDesign <- .vcov.matrix.lmm(structure = structure, data = data, index.cluster = outInit$index.cluster)
+    outDesign <- .vcov.matrix.lmm(structure = structure, data = data, index.cluster = outInit$index.cluster, drop.X = drop.X)
 
     structure$xfactor <- outDesign$xfactor
     structure$X <- list(var = outDesign$var,
@@ -418,7 +409,7 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
                                 index.clusterStrata = outInit$index.clusterStrata, U.strata = U.strata)
 
     ## ** prepare calculation of the score
-    if(precompute.moments){
+    if(precompute.moments && NCOL(X.mean)>0){
         if(is.na(var.weights[1])){
             wX.mean <- X.mean
             wY <- cbind(data[[var.outcome]])
@@ -426,6 +417,7 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
             wX.mean <- sweep(X.mean, FUN = "*", MARGIN = 1, STATS = sqrt(data[[var.weights[1]]]))
             wY <- cbind(data[[var.outcome]]*sqrt(data[[var.weights[1]]]))
         }
+
         precompute.XX <-  .precomputeXX(X = wX.mean, pattern = structure$X$Upattern$name, 
                                         pattern.ntime = stats::setNames(structure$X$Upattern$n.time, structure$X$Upattern$name),
                                         pattern.cluster = structure$X$Upattern$index.cluster, index.cluster = index.cluster)
@@ -440,7 +432,9 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
 
     ## ** pairs
     structure$X$pair.varcoef <- stats::setNames(lapply(structure$X$Upattern$name, function(iPattern){## iPattern <- structure$X$Upattern$name[1]
+
         iParamVar <- structure$X$Upattern$param[[iPattern]]
+        if(length(iParamVar)==0){return(NULL)}
 
         iOut <- .unorderedPairs(iParamVar)
         attr(iOut, "key") <- matrix(NA, nrow = length(iParamVar), ncol = length(iParamVar), dimnames = list(iParamVar,iParamVar))
@@ -458,6 +452,7 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
             iParamMu <- names(strata.mu)
         }
         iParamVar <- structure$X$Upattern$param[[iPattern]]
+        if(length(iParamVar)==0){return(NULL)}
         iOut <- unname(t(expand.grid(iParamMu, iParamVar)))
         return(iOut)
     }), structure$X$Upattern$name)
@@ -481,17 +476,21 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
                            }
                            return(name)
                        })
-    skeleton.param <- rbind(data.frame(name = colnames(X.mean),
-                                       strata = NA,
-                                       type = "mu",
-                                       level = gsub("^:","",gsub(":$","",mu.level)),
-                                       code = NA,
-                                       code.x = NA,
-                                       code.y = NA,
-                                       sigma = NA,
-                                       k.x = NA,
-                                       k.y = NA),
-                            structure$param)
+    if(NCOL(X.mean)>0){
+        skeleton.param <- rbind(data.frame(name = colnames(X.mean),
+                                           strata = NA,
+                                           type = "mu",
+                                           level = gsub("^:","",gsub(":$","",mu.level)),
+                                           code = NA,
+                                           code.x = NA,
+                                           code.y = NA,
+                                           sigma = NA,
+                                           k.x = NA,
+                                           k.y = NA),
+                                structure$param)
+    }else{
+        skeleton.param <- structure$param
+    }
     skeleton.param$fixed <- FALSE
     if(stratify.mean){
         skeleton.param$strata[skeleton.param$type=="mu"] <- strata.mu
@@ -518,7 +517,8 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
                 index.clusterStrata = index.clusterStrata,
                 time = list(n = max(unlist(index.clusterTime)), levels = levels(data$XXtimeXX), levels.original = NULL, nobs = table(unlist(index.clusterTime))),
                 cluster = list(n = length(index.cluster), levels = levels(data$XXclusterXX), levels.original = NULL, nobs = sapply(index.cluster,length)),
-                param = skeleton.param
+                param = skeleton.param,
+                drop.X = drop.X
                 )
     if(!is.na(var.weights[1])){
         out$weights <- data[[var.weights[1]]]
@@ -536,7 +536,7 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
 ## * helpers
 ## ** .model.matrix_regularize
 ## remove un-identifiable columns from the design matrix 
-.model.matrix_regularize <- function(formula, data, augmodel = FALSE, type){
+.model.matrix_regularize <- function(formula, data, augmodel = FALSE, type, drop.X){
 
     ## ## ** test 0: remove variable(s) with single level in the formula
     test.1value <- sapply(all.vars(formula),function(iVar){length(unique(data[[iVar]]))})
@@ -547,20 +547,23 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
 
     ## ** identify if there is an identifiability problem
     X <- stats::model.matrix(formula, data)
+    attr(X,"variable") <- all.vars(formula)
     X.qr <- qr(X)
 
     if(X.qr$rank==NCOL(X)){
-        if(augmodel){
+        if(NCOL(X)==0){
+            return(X)
+        }else if(augmodel){
             return(.augmodel.matrix(stats::delete.response(stats::terms(formula)),data))
         }else{
-            attr(X,"variable") <- all.vars(formula)
             return(X)
         }
     }else{
-        if(LMMstar.options()$drop.X==FALSE){
+        if(drop.X==FALSE){
             stop("The design matrix for the ",type," structure does not have full rank according to the QR decomposition. \n", sep = "")
         }
     }
+
     ## ** prepare
     tt <- stats::delete.response(stats::terms(formula))
     tt.factors <- attr(tt,"factors")
@@ -606,7 +609,7 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
     attrX$ls.level <- attrX$ls.level[index.keep]
     attrX$M.level <- attrX$M.level[index.keep,,drop=FALSE]
     attributes(X) <- c(attributes(X),attrX)
-    
+
     X.Mlevel <- attr(X,"M.level")
     X.reference <- attr(X,"reference")
 
@@ -641,18 +644,23 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
         X.old <- X
         test.keep <- colnames(X.old) %in% setdiff(colnames(X.old),rmX)
         X <- X.old[,test.keep,drop=FALSE]
-        if(qr(X)$rank==X.qr$rank){
-            message("Constant values in the design matrix for the ",type," structure.\n",
-                    "Coefficients \"",paste(unique(rmX), collapse = "\" \""),"\" relative to interactions \"",paste(names(ls.rmX), collapse = "\" \""),"\" have been removed. \n", sep = "")
+
+        txt <- paste0("Constant values in the design matrix for the ",type," structure.\n")
+        if(length(unique(rmX))==1){
+            txt <- paste0(txt, "Coefficient \"",paste(unique(rmX), collapse = "\" \""),"\" relative to interaction \"",paste(names(ls.rmX), collapse = "\" \""),"\" has been removed. \n")
         }else{
-            warning("Constant values in the design matrix for the ",type," structure.\n",
-                    "Coefficients \"",paste(unique(rmX), collapse = "\" \""),"\" relative to interactions \"",paste(names(ls.rmX), collapse = "\" \""),"\" have been removed. \n", sep = "")
+            txt <- paste0(txt, "Coefficients \"",paste(unique(rmX), collapse = "\" \""),"\" relative to interactions \"",paste(names(ls.rmX), collapse = "\" \""),"\" have been removed. \n")
+        }
+        if(qr(X)$rank==X.qr$rank){
+            message(txt)
+        }else{
+            warning(txt)
         }
         attr(X,"assign") <- attr(X.old,"assign")[test.keep] ## as.numeric(as.factor(attr(X.old,"assign")[test.keep])) - "(Intercept)" %in% colnames(X)
         attr(X,"contrasts") <- attr(X.old,"contrasts")
+        attr(X,"variable") <- attr(X.old,"variable")
         if(augmodel || X.qr$rank!=NCOL(X.qr$qr)){
             attr(X,"formula") <- attr(X.old,"formula")
-            attr(X,"variable") <- attr(X.old,"variable")
             attr(X,"term.labels") <- attr(X.old,"term.labels")[test.keep]
             attr(X,"order") <- attr(X.old,"order")[test.keep]
             attr(X,"ls.level") <- attr(X.old,"ls.level")[test.keep]
@@ -683,9 +691,9 @@ model.matrix.lmm <- function(object, data = NULL, effects = "mean", simplifies =
             X <- X.test
             attr(X,"assign") <- keep.attr$assign[iIndex]
             attr(X,"contrast") <- keep.attr$contrasts
+            attr(X,"variable") <- keep.attr$variable
             if(augmodel){
                 attr(X,"formula") <- keep.attr$formula
-                attr(X,"variable") <- keep.attr$variable
                 attr(X,"term.labels") <- keep.attr$term.labels[iIndex]
                 attr(X,"order") <- keep.attr$order[iIndex]
                 attr(X,"M.level") <- keep.attr$M.level[iIndex,,drop=FALSE]
