@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: sep  8 2021 (17:56) 
 ## Version: 
-## Last-Updated: jun 24 2022 (15:08) 
+## Last-Updated: nov  3 2022 (11:15) 
 ##           By: Brice Ozenne
-##     Update #: 2191
+##     Update #: 2346
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -91,10 +91,13 @@
 
     ## *** param rho
     if(NROW(X.cor)>0){
-        outRho <- .initRho(data = data, X.cor = X.cor, X.var = c(outK, list(strata.sigma = strata.sigma)), heterogeneous = structure$heterogeneous, 
+        outRho <- .initRho(data = data,
+                           X.cor = X.cor, X.var = c(outK, list(strata.sigma = strata.sigma)),
+                           heterogeneous = structure$heterogeneous, toeplitz = structure$type=="TOEPLITZ", block = structure$block,
                            U.cluster = U.cluster, index.cluster = index.cluster,
                            U.time = U.time, index.clusterTime = index.clusterTime, 
                            strata.var = strata.var, U.strata = U.strata, index.clusterStrata = index.clusterStrata, n.strata = n.strata)
+
         param.rho <- outRho$param[!duplicated(outRho$code)]
         strata.rho <- outRho$strata[!duplicated(outRho$code)]
         code.rho <- outRho$code[!duplicated(outRho$code)]
@@ -107,16 +110,18 @@
     }
 
     ## ** gather parameters
+    n.param <- length(param.sigma) + length(param.k) + length(param.rho)
+    
     structure$param <- data.frame(name = c(param.sigma,param.k,param.rho),
                                   strata = as.numeric(c(strata.sigma,strata.k,strata.rho)),
                                   type = c(rep("sigma",length=length(param.sigma)),rep("k",length=length(param.k)),rep("rho",length=length(param.rho))),
-                                  level = NA,
+                                  level = rep(NA, times = n.param),
                                   code = c(code.sigma,code.k,code.rho),
-                                  code.x = as.factor(NA),
-                                  code.y = as.factor(NA),
-                                  sigma = as.character(NA),
-                                  k.x = as.character(NA),
-                                  k.y = as.character(NA),                                  
+                                  code.x = rep(as.factor(NA), times = n.param),
+                                  code.y = rep(as.factor(NA), times = n.param),
+                                  sigma = rep(as.character(NA), times = n.param),
+                                  k.x = rep(as.character(NA), times = n.param),
+                                  k.y = rep(as.character(NA), times = n.param),                                  
                                   stringsAsFactors = FALSE)
     
     structure$param$level <- c(level.sigma,level.k,level.rho)
@@ -135,7 +140,7 @@
         ls.tempo <- tapply(factor(outRho$lp.y, levels = 1:length(attr(outRho,"levels")), labels = attr(outRho,"levels")), outRho$code, function(iX){iX}, simplify = FALSE)
         structure$param$code.y[names(ls.tempo)] <- ls.tempo
         names(structure$param$code.y) <- structure$param$name
-        
+
         structure$param[structure$param$type=="rho","sigma"] <- outRho$sigma[!duplicated(outRho$code)]
         structure$param[structure$param$type=="rho","k.x"] <- outRho$k.x[!duplicated(outRho$code)]
         structure$param[structure$param$type=="rho","k.y"] <- outRho$k.y[!duplicated(outRho$code)]
@@ -157,9 +162,108 @@
 ## * skeleton.CS
 .skeleton.CS <- .skeleton.IND
 
+## * skeleton.TOEPLITZ
+.skeleton.TOEPLITZ <- .skeleton.IND
+
 ## * skeleton.UN
 .skeleton.UN <- .skeleton.CS
 
+## * skeleton.EXP
+.skeleton.EXP <- function(structure, data, indexData = NULL){
+
+    ## ** prepare
+    if(is.null(indexData)){
+        indexData <- .extractIndexData(data = data, structure = structure)
+    }
+    strata.var <- structure$name$strata
+    U.cluster <- indexData$U.cluster
+    U.time <- indexData$U.time
+    U.strata <- indexData$U.strata
+    n.strata <- length(U.strata)
+    
+    index.clusterTime <- indexData$index.clusterTime ## list of index relative to the time at which the observations are made within cluster
+    index.clusterStrata <- indexData$index.clusterStrata ## vector of index relative to which strata each cluster belong to
+    index.cluster <- indexData$index.cluster ## list of positions of the observation belonging to each cluster in the dataset
+
+    X.var <- structure$X$var
+    X.cor <- structure$X$cor
+
+    ## ** param
+    ## *** sigma
+    outSigma <- .initSigma(X.var = X.var, strata.var = strata.var, U.strata = U.strata, n.strata = n.strata)
+    X.var <- outSigma$X    
+    param.sigma <- outSigma$param    
+    strata.sigma <- outSigma$strata    
+    code.sigma <- outSigma$code
+    level.sigma <- stats::setNames(as.list(outSigma$level),param.sigma)
+
+    ## *** k
+    outK <- .initK(X.var = X.var, strata.var = strata.var, U.strata = U.strata, n.strata = n.strata,
+                   param.sigma = param.sigma, strata.sigma = strata.sigma)
+    X.var <- outK$X    
+    param.k <- outK$param    
+    strata.k <- outK$strata    
+    code.k <- outK$code
+    level.k <- stats::setNames(as.list(outK$level),param.k)
+
+    ## *** param rho
+    regressor <- colnames(X.cor)[which(attr(X.cor, "assign") == max(attr(X.cor, "assign")))]
+
+    if(n.strata==1){
+        param.rho <- "lambda"
+        strata.rho <- 1
+        code.rho <- regressor
+        level.rho <- ""
+        if(structure$heterogeneous){
+            param.rho <- c(param.rho,"nugget")
+            strata.rho <- c(strata.rho,1)
+            code.rho <- c(code.rho,NA)
+            level.rho <- c(code.rho,"")
+        }
+    }else{
+        param.rho <- paste0("lambda",U.strata)
+        strata.rho <- 1:n.strata
+        code.rho <- regressor
+        level.rho <- U.strata
+        if(structure$heterogeneous){
+            param.rho <- c(param.rho,paste0("nugget",U.strata))
+            strata.rho <- c(strata.rho,1:n.strata)
+            code.rho <- c(code.rho,rep(NA, n.strata))
+            level.rho <- c(level.rho,U.strata)
+        }
+    }
+
+    ## ** gather parameters
+    n.param <- length(param.sigma) + length(param.k) + length(param.rho)
+    
+    structure$param <- data.frame(name = c(param.sigma,param.k,param.rho),
+                                  strata = as.numeric(c(strata.sigma,strata.k,strata.rho)),
+                                  type = c(rep("sigma",length=length(param.sigma)),rep("k",length=length(param.k)),rep("rho",length=length(param.rho))),
+                                  level = rep(NA, times = n.param),
+                                  code = c(code.sigma,code.k,code.rho),
+                                  code.x = rep(as.factor(NA), times = n.param),
+                                  code.y = rep(as.factor(NA), times = n.param),
+                                  sigma = rep(as.character(NA), times = n.param),
+                                  k.x = rep(as.character(NA), times = n.param),
+                                  k.y = rep(as.character(NA), times = n.param),                                  
+                                  stringsAsFactors = FALSE)
+    
+    structure$param$level <- c(level.sigma,level.k,level.rho)
+    attr(structure$param,"level.var") <- outSigma$code
+    if(any(structure$param$type=="k")){
+        structure$param[structure$param$type=="k","sigma"] <- names(strata.sigma)[structure$param[structure$param$type=="k","strata"]]
+        attr(structure$param,"level.var") <- c(attr(structure$param,"level.var"), outK$code)
+    }
+    structure$param <- structure$param[order(structure$param$strata),,drop=FALSE]
+    rownames(structure$param) <- NULL
+
+    ## ** export
+    structure$X$var <- X.var
+    structure$X$cor <- X.cor
+    return(structure)
+}
+
+
 ## * skeleton.CUSTOM
 .skeleton.CUSTOM <- function(structure, data, indexData = NULL){
 
@@ -203,6 +307,13 @@
 ## ** .initSigma
 .initSigma <- function(X.var, strata.var, U.strata, n.strata, sep = ":"){
 
+    if(NCOL(X.var)==0){
+        return(list(X = X.var,
+                    param = NULL,
+                    strata = NULL,
+                    code = NULL,
+                    level = NULL))
+    }
 
     if(n.strata==1){
         param.sigma <- "sigma"
@@ -297,7 +408,8 @@
 
 ## ** .initRho
 ## for each cluster compute all pairwise difference in covariates to find the parameters
-.initRho <- function(data, X.cor, X.var, heterogeneous, 
+.initRho <- function(data, X.cor, X.var,
+                     heterogeneous, toeplitz, block,
                      U.cluster, index.cluster,
                      U.time, index.clusterTime, 
                      strata.var, U.strata, index.clusterStrata, n.strata, sep = c(":")){
@@ -323,6 +435,14 @@
 
     ## *** identify the correlation parameters
     out <- vector(mode = "list", length = n.strata)
+    if(toeplitz){
+        index.XcolTime <- which(attr(X.cor,"term.labels")==attr(X.cor,"variable")[1])
+        if(block){
+            index.XcolBlock <- which(attr(X.cor,"term.labels")==attr(X.cor,"variable")[2])
+        }else{
+            index.XcolBlock <- NULL
+        }
+    }
 
     for(iStrata in 1:n.strata){
 
@@ -367,140 +487,132 @@
         }), iCluster)
 
         ## **** remove cluster with single value or identical to other cluster in term of linear predictors
-        iIndex.unique <- intersect(which(!duplicated(iULpCluster.cor)),
-                                   which(sapply(iIndex.cluster,length)>1))
+        iIndex.Nsingle <- which(sapply(iIndex.cluster,length)>1)
+        iIndex.unique <- iIndex.Nsingle[which(!duplicated(iULpCluster.cor[iIndex.Nsingle]))]
+                                   
         iCluster2 <- iCluster[iIndex.unique]
 
         iN.pair <- unique(sapply(iULpIndex.cor[iIndex.unique], length))
         ls.pair <- vector(mode = "list", length = max(iN.pair))
         ls.pair[iN.pair] <- lapply(iN.pair, function(iN){.unorderedPairs(1:iN, distinct = TRUE)})
-        
+
         ## **** contrast all pairs
         for(iC in iCluster2){ ## iC <- iCluster2[1]
             iCindex  <- index.cluster[[iC]][iULpIndex.cor[[iC]]]
             iCX.cor <- X.cor[iCindex,,drop=FALSE]
             iData <- data[iCindex,,drop=FALSE]
-
-            if(NROW(iCX.cor)==1){ ## SAME LINEAR PREDICTOR FOR ALL OBSERVATIONS WITHIN CLUSTER
-                if(heterogeneous){ ## make sure it is compatible with the next case (in case some cluster have only a single pair of observations)
-                    iDF.diff <- as.data.frame(cbind("R",iCX.cor), drop = TRUE)
-                }else{
-                    iDF.diff <- as.data.frame(matrix(c("R",rep(iStrata,NCOL(iCX.cor))), nrow = 1, ncol = 1+NCOL(iCX.cor)))
-                }
-                iCode <- as.character(interaction(iDF.diff, drop=TRUE))
-                ## name difference according to the covariate values
-                if(length(attr(X.cor,"M.level"))==0 || identical(names(attr(X.cor,"M.level")),strata.var)){
-                    if(n.strata>1){
-                        iLevel <- paste0(":",U.strata[iStrata])
-                    }else{
-                        iLevel <- ""
-                    }
-                }else{
-                    iLevel <- paste0("(",as.character(interaction(iData[,names(attr(X.cor,"M.level")),drop=FALSE],drop=TRUE)),")")
-                    if(n.strata>1){
-                        iLevel <- paste0(iLevel,":",U.strata[iStrata])
-                    }
-                }
-                iName <- paste0("rho",iLevel)
-
-                iOut <- data.frame(lp.x = lpnCluster.cor[[iC]][1],
-                                   lp.y = lpnCluster.cor[[iC]][2],
-                                   strata = iStrata,
-                                   code = iCode,
-                                   level = iLevel,
-                                   param = iName,
-                                   sigma = NA,
-                                   k.x = NA,
-                                   k.y = NA)
-
-                if(!missing(X.var)){ ## add corresponding variance parameters
-                    iCX.sigma <- iX.sigma[index.cluster[[iC]][1:2],,drop=FALSE]
-                    iOut$sigma <- colnames(iCX.sigma)[colSums(iCX.sigma)!=0]
-
-                    if(length(iX.k)>0){
-                        iCX.k <- iX.k[index.cluster[[iC]][1:2],,drop=FALSE]
-                        if(sum(iCX.k[1,,drop=FALSE]!=0)){
-                            iOut$k.x <- colnames(iCX.k)[iCX.k[1,,drop=FALSE]==1]
-                        }
-                        if(sum(iCX.k[2,,drop=FALSE]!=0)){
-                            iOut$k.y <- colnames(iCX.k)[iCX.k[2,,drop=FALSE]==1]
-                        }
-                    }
-                }
-                out[[iStrata]] <- rbind(out[[iStrata]], iOut)                   
-                    
-            }else{ ## DIFFERENT LINEAR PREDICTORS WITHIN CLUSTER
+            if(NROW(iCX.cor)==1){
+                iPair.time <- matrix(1, nrow = 2, ncol = 1)
+            }else{
                 iPair.time <- ls.pair[[NROW(iCX.cor)]]
-                iM <- matrix(lpnCluster.cor[[iC]][iULpIndex.cor[[iC]]][iPair.time], ncol = 2, byrow = TRUE, dimnames = list(NULL, c("x","y")))
-                
-                iDF.diff <- as.data.frame(do.call(rbind,lapply(1:NCOL(iPair.time),function(iCol){ ## iCol <- 1
+            }
+            iM <- matrix(lpnCluster.cor[[iC]][iULpIndex.cor[[iC]]][iPair.time], ncol = 2, byrow = TRUE, dimnames = list(NULL, c("x","y")))
 
-                    if(iM[iCol,"x"] < iM[iCol,"y"]){
-                        ## make sure same correlation coefficient for (X=1,X=0) and (X=0,X=1)
-                        iCX.cor1 <- iCX.cor[iPair.time[1,iCol],,drop=FALSE]
-                        iCX.cor2 <- iCX.cor[iPair.time[2,iCol],,drop=FALSE]
-                    }else if(iM[iCol,"x"] > iM[iCol,"y"]){
-                        ## make sure same correlation coefficient for (X=1,X=0) and (X=0,X=1)
-                        iCX.cor1 <- iCX.cor[iPair.time[2,iCol],,drop=FALSE]
-                        iCX.cor2 <- iCX.cor[iPair.time[1,iCol],,drop=FALSE]
+            iDF.diff <- as.data.frame(do.call(rbind,lapply(1:NCOL(iPair.time),function(iCol){ ## iCol <- 1
+
+                if(iM[iCol,"x"] < iM[iCol,"y"]){
+                    ## make sure same correlation coefficient for (X=1,X=0) and (X=0,X=1)
+                    iCX.cor1 <- iCX.cor[iPair.time[1,iCol],,drop=FALSE]
+                    iCX.cor2 <- iCX.cor[iPair.time[2,iCol],,drop=FALSE]
+                }else if(iM[iCol,"x"] > iM[iCol,"y"]){
+                    ## make sure same correlation coefficient for (X=1,X=0) and (X=0,X=1)
+                    iCX.cor1 <- iCX.cor[iPair.time[2,iCol],,drop=FALSE]
+                    iCX.cor2 <- iCX.cor[iPair.time[1,iCol],,drop=FALSE]
+                }else{
+                    iCX.cor1 <- iCX.cor[min(iPair.time[,iCol]),,drop=FALSE]
+                    iCX.cor2 <- iCX.cor[max(iPair.time[,iCol]),,drop=FALSE]
+                }
+
+                if(toeplitz){
+                    if(block){
+                        if(iCX.cor2[,index.XcolBlock] == iCX.cor1[,index.XcolBlock]){ ## same block
+                            if(heterogeneous=="UN"){
+                                return(cbind("R",iCX.cor2[,index.XcolBlock],paste(iCX.cor1,collapse=""),iCX.cor2[,index.XcolTime]-iCX.cor1[,index.XcolTime]))
+                            }else if(heterogeneous=="LAG"){
+                                return(cbind("R",iCX.cor2[,index.XcolBlock],abs(iCX.cor2[,index.XcolTime]-iCX.cor1[,index.XcolTime])))
+                            }else if(heterogeneous=="CS"){
+                                return(cbind("R",iCX.cor2[,index.XcolBlock],0))
+                            }
+                        }else{ ## different block
+                            if(heterogeneous=="UN"){
+                                if(iCX.cor2[,index.XcolTime]==iCX.cor1[,index.XcolTime]){
+                                    return(cbind("D",abs(iCX.cor2[,index.XcolBlock]-iCX.cor1[,index.XcolBlock]),"0",iCX.cor2[,index.XcolTime]-iCX.cor1[,index.XcolTime]))
+                                }else{
+                                    return(cbind("D",abs(iCX.cor2[,index.XcolBlock]-iCX.cor1[,index.XcolBlock]),paste(iCX.cor1,collapse=""),iCX.cor2[,index.XcolTime]-iCX.cor1[,index.XcolTime]))
+                                }                                
+                            }else if(heterogeneous=="LAG"){
+                                return(cbind("D",abs(iCX.cor2[,index.XcolBlock]-iCX.cor1[,index.XcolBlock]),abs(iCX.cor2[,index.XcolTime]-iCX.cor1[,index.XcolTime])))
+                            }else if(heterogeneous=="CS"){
+                                return(cbind("D",abs(iCX.cor2[,index.XcolBlock]-iCX.cor1[,index.XcolBlock]),as.numeric(iCX.cor2[,index.XcolTime]!=iCX.cor1[,index.XcolTime])))
+                            }
+                        }
                     }else{
-                        iCX.cor1 <- iCX.cor[min(iPair.time[,iCol]),,drop=FALSE]
-                        iCX.cor2 <- iCX.cor[max(iPair.time[,iCol]),,drop=FALSE]
+                        return(cbind("D",abs(iCX.cor2[,index.XcolTime]-iCX.cor1[,index.XcolTime])))
                     }
-                    if(heterogeneous){
+
+                }else{
+                    if(heterogeneous>=1){
                         if(all(iCX.cor1==iCX.cor2)){return(cbind("R",iCX.cor1))}else{return(cbind(paste0("D",paste(iCX.cor1,collapse="")),iCX.cor2-iCX.cor1))}
                     }else{
                         if(all(iCX.cor1==iCX.cor2)){return(matrix(c("R",rep(iStrata,NCOL(iCX.cor1))), nrow = 1, ncol = 1+NCOL(iCX.cor1)))}else{return(matrix(c("D",as.numeric(iCX.cor2!=iCX.cor1)), nrow = 1))}
                     }
-                })))
-                iCode <- as.character(interaction(iDF.diff, drop=TRUE))
-                ## name difference according to the covariate values
-                iName.covcor <- setdiff(names(attr(X.cor,"M.level")),strata.var)
-                iCov <- as.character(interaction(iData[,iName.covcor,drop=FALSE],drop=TRUE))
 
+                    }
+            })))
+
+            iCode <- as.character(interaction(iDF.diff, drop=TRUE))
+            ## name difference according to the covariate values
+            iName.covcor <- setdiff(names(attr(X.cor,"M.level")),strata.var)
+            if(length(iName.covcor)>0){
+                iCov <- as.character(interaction(iData[,iName.covcor,drop=FALSE],drop=TRUE))
                 index.iUCode <- which(!duplicated(iCode))
                 iULevel <- stats::setNames(sapply(index.iUCode,function(iCol){
                     return(paste0("(",paste(unique(c(iCov[min(iPair.time[,iCol])],iCov[max(iPair.time[,iCol])])),collapse=","),")"))
                 }), iCode[index.iUCode])
                 iLevel <- iULevel[iCode]
-                                
                 if(n.strata>1){
                     iLevel <- paste0(iLevel,":",U.strata[iStrata])
                 }
-                iName <- paste0("rho",iLevel)
+            }else if(n.strata==1){
+                iLevel <- ""
+            }else if(n.strata>1){
+                iLevel <- paste0(":",U.strata[iStrata])
+            }
+                                
+            iName <- paste0("rho",iLevel)
 
-                index.unique <- which(!duplicated(cbind(iCode, iM)))
-                iOut <- data.frame(lp.x = iM[index.unique,"x"],
-                                   lp.y = iM[index.unique,"y"],
-                                   strata = iStrata,
-                                   code = iCode[index.unique],
-                                   level = iLevel[index.unique],
-                                   param = iName[index.unique],
-                                   sigma = NA,
-                                   k.x = NA,
-                                   k.y = NA)
+            index.unique <- which(!duplicated(cbind(iCode, iM)))
+            iOut <- data.frame(lp.x = iM[index.unique,"x"],
+                               lp.y = iM[index.unique,"y"],
+                               strata = iStrata,
+                               code = iCode[index.unique],
+                               level = iLevel[index.unique],
+                               param = iName[index.unique],
+                               sigma = NA,
+                               k.x = NA,
+                               k.y = NA)
 
-                if(!missing(X.var)){ ## add corresponding variance parameters
-                    iCX.sigma <- iX.sigma[iCindex,,drop=FALSE]
+            if(!missing(X.var)){ ## add corresponding variance parameters
+                iCX.sigma <- iX.sigma[iCindex,,drop=FALSE]
+                if(NCOL(iCX.sigma)>0){
                     iOut$sigma <- colnames(iCX.sigma)[colSums(iCX.sigma)!=0]
-
-                    if(length(iX.k)>0){
-                        iCX.k <- iX.k[iCindex,,drop=FALSE]
-                        iNames.k <- colnames(iCX.k)
-                        iName2 <- sapply(index.unique,function(iCol){ ## iCol <- 4
-                            iName.x <- iNames.k[which(iCX.k[min(iPair.time[,iCol]),]==1)]
-                            if(length(iName.x)==0){iName.x <- NA}
-                            iName.y <- iNames.k[which(iCX.k[max(iPair.time[,iCol]),]==1)]
-                            if(length(iName.y)==0){iName.y <- NA}
-                            return(c(iName.x,iName.y))
-                        })
-                        iOut$k.x <- iName2[1,]
-                        iOut$k.y <- iName2[2,]
-                    }
                 }
-                out[[iStrata]] <- rbind(out[[iStrata]], iOut)
-            
-            }            
+
+                if(length(iX.k)>0){
+                    iCX.k <- iX.k[iCindex,,drop=FALSE]
+                    iNames.k <- colnames(iCX.k)
+                    iName2 <- sapply(index.unique,function(iCol){ ## iCol <- 4
+                        iName.x <- iNames.k[which(iCX.k[min(iPair.time[,iCol]),]==1)]
+                        if(length(iName.x)==0){iName.x <- NA}
+                        iName.y <- iNames.k[which(iCX.k[max(iPair.time[,iCol]),]==1)]
+                        if(length(iName.y)==0){iName.y <- NA}
+                        return(c(iName.x,iName.y))
+                    })
+                    iOut$k.x <- iName2[1,]
+                    iOut$k.y <- iName2[2,]
+                }
+            }
+            out[[iStrata]] <- rbind(out[[iStrata]], iOut)
         }
         out[[iStrata]] <- out[[iStrata]][!duplicated(out[[iStrata]]),,drop=FALSE]
     }
@@ -513,6 +625,7 @@
     return(out)
 }
 
+
 ## ** .colnameOrder
 ## reorder the variable in the column name
 .colnameOrder <- function(X, strata.var, n.strata){
